import { Status } from "../../DataFrame";
import { IntRange } from "../../Helpers";
import { IDevice } from "../Devices/IDevice";
import { ResponseResult } from "../FieldChangeResult";
import { ICustomUpdateActions } from "./CustomUpdateActions/ICustomUpdateActions";
import { ControllerUpdatesInfo, ControllerVersion, IDeviceUpdateInfo } from "./IUpdateInfo";
export interface IUpdatesService {
    GetServiceName(): string;
    /**
     * Checks available controller updates
     */
    CheckControllerUpdatesAsync(): Promise<ControllerUpdatesInfo>;
    /**
     * Checks available device update
     * @param deviceGuid
     */
    CheckDeviceUpdateAsync(deviceGuid: string): Promise<IDeviceUpdateInfo | ResponseResult<UpdateErrorCode>>;
    /**
     * Checks available device update
     * @param device
     */
    CheckDeviceUpdateAsync(device: IDevice): Promise<IDeviceUpdateInfo | ResponseResult<UpdateErrorCode>>;
    /**
     * Function to install single controller update, before use this function we need to use CheckControllerUpdatesAsync() to check if any updates are available.
     * @param updateIdentifier update guid - present in ControllerUpdatesInfo object
     * @param updateProgress callback function which is called when recived information about update progress from controller
     * @param updateAction custom actions to do (may be protocol-dependent or may not occur)
     * @returns Status.OK when update is finished (regardless of UpdateProgress result, the update result is given in the callback)
     */
    InstallControllerUpdateAsync<T extends ICustomUpdateActions>(updateGuid: string, updateProgress: UpdateProgress, updateAction?: UpdateAction<T>): Promise<Status.OK | ResponseResult<UpdateErrorCode>>;
    /**
     * Function to install all controller updates, before use this function we need to use CheckControllerUpdatesAsync() to check if any updates are available.
     * @param updateProgress callback function which is called when recived information about update progress from controller
     * @param updateAction custom actions to do (may be protocol-dependent or may not occur)
     * @returns Status.OK when update is finished (regardless of UpdateProgress result, the update result is given in the callback)
     */
    InstallControllerUpdateAsync<T extends ICustomUpdateActions>(updateProgress: UpdateProgress, updateAction?: UpdateAction<T>): Promise<Status.OK | ResponseResult<UpdateErrorCode>>;
    /**
     * Installs a device update if available
     * @param device IDevice object
     * @param updateProgress callback function which is called when recived information about update progress from controller
     * @param updateAction custom actions to do (may be protocol-dependent or may not occur)
     * @returns Status.OK when update is finished (regardless of UpdateProgress result, the update result is given in the callback)
     * When any update is not finished with success, method returns UpdatesPartiallyInstalled error code.
     */
    InstallDeviceUpdateAsync<T extends ICustomUpdateActions>(device: IDevice, updateProgress: UpdateProgress, updateAction?: UpdateAction<T>): Promise<Status.OK | ResponseResult<UpdateErrorCode>>;
    /**
     * Installs a device update if available
     * @param deviceGuid device Guid
     * @param updateProgress callback function which is called when recived information about update progress from controller
     * @param updateAction custom actions to do (may be protocol-dependent or may not occur)
     * @returns Status.OK when update is finished (regardless of UpdateProgress result, the update result is given in the callback)
     */
    InstallDeviceUpdateAsync<T extends ICustomUpdateActions>(deviceGuid: string, updateProgress: UpdateProgress, updateAction?: UpdateAction<T>): Promise<Status.OK | ResponseResult<UpdateErrorCode>>;
    /**
     * The function checks the version of all components available on the controller (radio, software, etc.).
     */
    CheckControllerVersionAsync(): Promise<ControllerVersion[] | ResponseResult<UpdateErrorCode>>;
    /**
     * Function returns current software version.
     * @param deviceGuid
     */
    CheckDeviceVersionAsync(deviceGuid: string): Promise<string | ResponseResult<UpdateErrorCode>>;
    /**
     * Function returns current software versions (radio, software, etc.).
     * @param device
     */
    CheckDeviceVersionAsync(device: IDevice): Promise<string | ResponseResult<UpdateErrorCode>>;
}
export declare type UpdateProgress = (updateProgress: UpdateProgressInfo) => void;
export declare type UpdateAction<T extends ICustomUpdateActions> = (updateAction: T) => any;
export declare type Progress = IntRange<0, 101>;
export declare enum UpdateStatus {
    DownloadingFailure = -5,
    InstallationStopped = -4,
    InstallationError = -3,
    IncompatiblePackage = -2,
    Error = -1,
    OK = 0,
    DownloadingPackage = 1,
    CheckingPackage = 2,
    PreparingForUpgrade = 3,
    Upgrading = 4,
    MovingData = 5,
    Reloading = 6,
    Upgraded = 7
}
export declare enum UpdateState {
    WaitingForUser = 1,
    Downloading = 2,
    Unpackaging = 3,
    CheckingApplicability = 4,
    MigratingData = 5,
    CreatingBackup = 6,
    Deploying = 7,
    Installing = 8,
    Removing = 9,
    Error = 10,
    ReadyToDownload = 11,
    PreparingSystem = 12,
    CheckingPackageIntegrity = 13,
    CheckingIntegrity = 14,
    UndoingInstallation = 15,
    Installed = 16
}
export declare enum UserInput {
    Ignore = 0,
    Cancel = 1,
    Install = 2
}
export declare class UpdateProgressInfo {
    UpdatedResourceName: string;
    Percentage: Progress;
    Status: UpdateStatus;
    UpdateNumber: number;
    AvailableUpdates: number;
}
export declare enum UpdateErrorCode {
    DeviceIsUpToDate = 0,
    CannotFindDevice = 1,
    UnsupportedApiVersion = 2,
    UnsupportedDevice = 3,
    CannotGetFirmwareVersion = 4,
    CannotGetUpdates = 5,
    CannotProcessUpdate = 6,
    UpdateActionsNotSupported = 7,
    UnsupportedExtensionVersion = 8,
    UpdatesPartiallyInstalled = 9
}
